package transport

import (
	"net/http"
	"strconv"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminDashboardDosbingHandler struct {
	dashboardDosbingUseCase usecase.DashboardDosbingUseCase
}

func NewAdminDashboardDosbingHandler(cfg config.DashboardDosbingTransportConfig) *AdminDashboardDosbingHandler {
	dashboardDosbingRepository := repository.NewDashboardDosbingRepository(cfg.DBWrite, cfg.DBRead)
	dashboardDosbingUsecase := usecase.NewDashboardDosbingUsecase(dashboardDosbingRepository)

	return &AdminDashboardDosbingHandler{
		dashboardDosbingUseCase: dashboardDosbingUsecase,
	}
}

func (t *AdminDashboardDosbingHandler) MountAdmin(group *echo.Group) {
	group.GET("/dashboard/dosbing", t.GetMahasiswaByDosenId)
	group.GET("/dashboard/dosbing/statistic", t.GetAmountMahasiswaByDosenId)
	group.GET("/dashboard/dosbing/status-bimbingan", t.GetAmountStatusBimbinganMahasiswaByDosenId)
}

// GetMahasiswaByDosenId retrieves a list of mahasiswa supervised by a specific dosen for TA.
// @Summary Get mahasiswa by dosen ID
// @Description Retrieves a list of mahasiswa supervised by a specific dosen for TA.
// @Tags Dashboard Dosbing
// @Accept json
// @Produce json
// @Param id_dosen query string true "Dosen ID"
// @Param limit query string false "Limit"
// @Param offset query string false "Offset"
// @Success 200 {object} []entity.MahasiswaBimbingan "List mahasiswa successfully retrieved"
// @Router /api/admin/dashboard/dosbing [get]
func (t *AdminDashboardDosbingHandler) GetMahasiswaByDosenId(c echo.Context) error {
	idDosen := c.QueryParam("id_dosen")
	sizeStr := c.QueryParam("limit")
	offsetStr := c.QueryParam("offset")

	if idDosen == "" {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "insert id_dosen", nil))
	}

	size, err := strconv.Atoi(sizeStr)
	if err != nil {
		// Handle error, maybe return an error response
		size = 10
	}

	offset, err := strconv.Atoi(offsetStr)
	if err != nil {
		// Handle error, maybe return an error response
		offset = 0
	}

	listMahasiswa, err := t.dashboardDosbingUseCase.GetMahasiswaByDosenId(idDosen, offset, size)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "List mahasiswa successfully retreived", listMahasiswa))

}

// GetAmountMahasiswaByDosenId retrieves the amount of mahasiswa supervised by a specific dosen for TA.
// @Summary Get amount of mahasiswa by dosen ID
// @Description Retrieves the amount of mahasiswa supervised by a specific dosen for TA.
// @Tags Dashboard Dosbing
// @Accept json
// @Produce json
// @Param id_dosen query string true "Dosen ID"
// @Success 200 {object} []entity.JumlahMahasiswa "List mahasiswa successfully retreived"
// @Router /api/admin/dashboard/dosbing/statistic [get]
func (t *AdminDashboardDosbingHandler) GetAmountMahasiswaByDosenId(c echo.Context) error {
	idDosen := c.QueryParam("id_dosen")

	if idDosen == "" {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "insert id_dosen", nil))
	}

	listMahasiswa, err := t.dashboardDosbingUseCase.GetAmountMahasiswaByDosenId(idDosen)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "List mahasiswa successfully retreived", listMahasiswa[0]))

}

// GetAmountStatusBimbinganMahasiswaByDosenId retrieves the amount of mahasiswa's bimbingan status supervised by a specific dosen for TA.
// @Summary Get amount of mahasiswa's bimbingan status by dosen ID
// @Description Retrieves the amount of mahasiswa's bimbingan status supervised by a specific dosen for TA.
// @Tags Dashboard Dosbing
// @Accept json
// @Produce json
// @Param id_dosen query string true "Dosen ID"
// @Success 200 {object} []entity.StatusBimbinganMahasiswa "List mahasiswa successfully retreived"
// @Router /api/admin/dashboard/dosbing/status-bimbingan [get]
func (t *AdminDashboardDosbingHandler) GetAmountStatusBimbinganMahasiswaByDosenId(c echo.Context) error {
	idDosen := c.QueryParam("id_dosen")

	if idDosen == "" {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "insert id_dosen", nil))
	}

	listMahasiswa, err := t.dashboardDosbingUseCase.GetAmountStatusBimbinganMahasiswaByDosenId(idDosen)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "List mahasiswa successfully retreived", listMahasiswa[0]))

}
