package transport

import (
	"fmt"
	"net/http"
	"strconv"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
	"gopkg.in/gomail.v2"
)

type AdminJadwalSeminarHandler struct {
	jadwalSeminarUsecase usecase.JadwalSeminarUsecase
}

func NewAdminJadwalSeminarHandler(cfg config.JadwalSeminarTransportConfig) *AdminJadwalSeminarHandler {
	jadwalSeminarRepository := repository.NewJadwalSeminarRepository(cfg.DBWrite, cfg.DBRead)
	jadwalSeminarUsecase := usecase.NewJadwalSeminarUseCase(jadwalSeminarRepository)

	return &AdminJadwalSeminarHandler{
		jadwalSeminarUsecase: jadwalSeminarUsecase,
	}

}

func (t *AdminJadwalSeminarHandler) MountAdmin(group *echo.Group) {
	group.POST("/jadwal-seminar", t.AddJadwalSeminar)
	group.GET("/jadwal-seminar-batch", t.GetJadwalSeminarByBatch)
	group.DELETE("/jadwal-seminar", t.DeleteJadwalSeminar)
	group.PATCH("/jadwal-seminar", t.EditJadwalSeminar)
	group.GET("/jadwal-seminar", t.GetJadwalSeminarAll)
	group.GET("/jadwal-seminar-range", t.GetJadwalSeminarInDateRange)
}

// AddJadwalSeminar adds a new jadwal seminar record.
// @Summary Add jadwal seminar
// @Description Adds a new jadwal seminar record.
// @Tags Jadwal Seminar
// @Accept json
// @Produce json
// @Param body body entity.JadwalSeminarRequest true "Jadwal Seminar request body"
// @Success 201 {object} entity.JadwalSeminar "Jadwal added successfully"
// @Router /jadwal-seminar [post]
func (t *AdminJadwalSeminarHandler) AddJadwalSeminar(c echo.Context) error {
	request := new(entity.JadwalSeminarRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	targetJadwalSeminar := &entity.JadwalSeminar{
		Name:        request.Name,
		Nim:         request.Nim,
		NamaDosbing: request.NamaDosbing,
		ScheduledAt: request.ScheduledAt,
		NamaDosuji1: request.NamaDosuji1,
		Ruang:       request.Ruang,
		Judul:       request.Judul,
		IDBatch:     request.IDBatch,
	}

	fmt.Println(targetJadwalSeminar)

	newJadwalSeminar, err := t.jadwalSeminarUsecase.AddJadwalSeminar(*targetJadwalSeminar)

	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	subject := "Penambahan Jadwal Seminar pada GraduIT"
	// headers := "MIME-version: 1.0;\nContent-Type: text; charset=\"UTF-8\";"
	bodyText := "Dengan surel ini, diberitahukan bahwa telah dilakukan penambahan jadwal seminar pada aplikasi GraduIT dengan informasi:\nNama:" + request.Name + "\nNIM: " + request.Nim + "\nNama Dosbing:" + request.NamaDosbing
	smtpHost := "smtp.gmail.com"
	smtpUsername := "graduititb@gmail.com"
	smtpPassword := "noim hyum imnm rgxw"

	m := gomail.NewMessage()
	m.SetHeader("From", "graduititb@gmail.com")
	m.SetHeader("To", "maximun154@gmail.com")
	m.SetHeader("Subject", subject)
	m.SetBody("text", bodyText)

	d := gomail.NewDialer(smtpHost, 587, smtpUsername, smtpPassword)

	err2 := d.DialAndSend(m)
	if err2 != nil {
		fmt.Println(err2)
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Jadwal added successfully", newJadwalSeminar))
}

// GetJadwalSeminarByBatch retrieves jadwal seminar records by batch ID.
// @Summary Get jadwal seminar by batch
// @Description Retrieves jadwal seminar records by batch ID.
// @Tags Jadwal Seminar
// @Accept json
// @Produce json
// @Param id_batch query string true "Batch ID"
// @Success 200 {object} []entity.JadwalSeminar "Jadwal Seminar successfully retreived"
// @Router /jadwal-seminar-batch [get]
func (t *AdminJadwalSeminarHandler) GetJadwalSeminarByBatch(c echo.Context) error {
	idBatch := c.QueryParam("id_batch")
	sizeStr := c.QueryParam("limit")
	offsetStr := c.QueryParam("offset")

	size, err := strconv.Atoi(sizeStr)
	if err != nil {
		// Handle error, maybe return an error response
		size = 10
	}

	offset, err := strconv.Atoi(offsetStr)
	if err != nil {
		// Handle error, maybe return an error response
		offset = 0
	}
	jadwal, err := t.jadwalSeminarUsecase.GetJadwalSeminarByBatch(idBatch, size, offset)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Seminar successfully retreived", jadwal))
}

// DeleteJadwalSeminar deletes a jadwal seminar record by ID.
// @Summary Delete jadwal seminar
// @Description Deletes a jadwal seminar record by ID.
// @Tags Jadwal Seminar
// @Accept json
// @Produce json
// @Param body body entity.EditJadwalSeminarRequest true "Edit Jadwal Seminar request body"
// @Success 200 {object} entity.JadwalSeminar "Jadwal Seminar successfully deleted"
// @Router /jadwal-seminar [delete]
func (t *AdminJadwalSeminarHandler) DeleteJadwalSeminar(c echo.Context) error {
	request := new(entity.EditJadwalSeminarRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	jadwal, err := t.jadwalSeminarUsecase.DeleteJadwalSeminar(request.ID)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Seminar successfully deleted", jadwal))
}

// EditJadwalSeminar edits an existing jadwal seminar record.
// @Summary Edit jadwal seminar
// @Description Edits an existing jadwal seminar record.
// @Tags Jadwal Seminar
// @Accept json
// @Produce json
// @Param body body entity.EditJadwalSeminarRequest true "Edit Jadwal Seminar request body"
// @Success 200 {object} entity.JadwalSeminar "Jadwal Seminar successfully edited"
// @Router /jadwal-seminar [patch]
func (t *AdminJadwalSeminarHandler) EditJadwalSeminar(c echo.Context) error {
	request := new(entity.EditJadwalSeminarRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	fmt.Println(request)

	targetJadwalSeminar := &entity.JadwalSeminar{
		ID:          request.ID,
		Name:        request.Name,
		Nim:         request.Nim,
		NamaDosbing: request.NamaDosbing,
		ScheduledAt: request.ScheduledAt,
		NamaDosuji1: request.NamaDosuji1,
		Ruang:       request.Ruang,
		Judul:       request.Judul,
		IDBatch:     request.IDBatch,
	}

	// Call the update method from the use case passing the targetJadwalSeminar
	updatedJadwalSeminar, err := t.jadwalSeminarUsecase.EditJadwalSeminar(*targetJadwalSeminar)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Seminar successfully edited", updatedJadwalSeminar))
}

// GetJadwalSeminarAll retrieves all jadwal seminar records.
// @Summary Get all jadwal seminar
// @Description Retrieves all jadwal seminar records.
// @Tags Jadwal Seminar
// @Accept json
// @Produce json
// @Success 200 {object} []entity.JadwalSeminar "Jadwal Seminar successfully retreived"
// @Router /jadwal-seminar [get]
func (t *AdminJadwalSeminarHandler) GetJadwalSeminarAll(c echo.Context) error {
	jadwal, err := t.jadwalSeminarUsecase.GetJadwalSeminarAll()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Seminar successfully retreived", jadwal))
}

// GetJadwalSeminarInDateRange retrieves all jadwal seminar records in a date range.
// @Summary Get all jadwal seminar within a date range
// @Description Retrieves all jadwal seminar records within a date range.
// @Tags Jadwal Seminar
// @Accept json
// @Produce json
// @Success 200 {object} []entity.JadwalSeminar
// @Router /jadwal-seminar-range [get]
func (t *AdminJadwalSeminarHandler) GetJadwalSeminarInDateRange(c echo.Context) error {
	startDate := c.QueryParam("startDate")
	endDate := c.QueryParam("endDate")

	jadwal, err := t.jadwalSeminarUsecase.GetJadwalSeminarInDateRange(startDate, endDate)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Seminar successfully retreived", jadwal))
}
