package transport

import (
	"fmt"
	"net/http"
	"strconv"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminJadwalSidangHandler struct {
	jadwalSidangUsecase usecase.JadwalSidangUsecase
}

func NewAdminJadwalSidangHandler(cfg config.JadwalSidangTransportConfig) *AdminJadwalSidangHandler {
	jadwalSidangRepository := repository.NewJadwalSidangRepository(cfg.DBWrite, cfg.DBRead)
	jadwalSidangUsecase := usecase.NewJadwalSidangUseCase(jadwalSidangRepository)

	return &AdminJadwalSidangHandler{
		jadwalSidangUsecase: jadwalSidangUsecase,
	}

}

func (t *AdminJadwalSidangHandler) MountAdmin(group *echo.Group) {
	group.POST("/jadwal-sidang", t.AddJadwalSidang)
	group.GET("/jadwal-sidang-batch", t.GetJadwalSidangByBatch)
	group.GET("/jadwal-sidang", t.GetJadwalSidangAll)
	group.DELETE("/jadwal-sidang", t.DeleteJadwalSidang)
	group.PATCH("/jadwal-sidang", t.EditJadwalSidang)
	group.GET("/jadwal-sidang-range", t.GetJadwalSidangInDateRange)
}

// AddJadwalSidang adds a new jadwal sidang record.
// @Summary Add jadwal sidang
// @Description Adds a new jadwal sidang record.
// @Tags Jadwal Sidang
// @Accept json
// @Produce json
// @Param request body entity.JadwalSidangRequest true "Jadwal Sidang Request"
// @Success 201 {object} entity.JadwalSidang
// @Router /jadwal-sidang [post]
func (t *AdminJadwalSidangHandler) AddJadwalSidang(c echo.Context) error {
	request := new(entity.JadwalSidangRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	targetJadwalSidang := &entity.JadwalSidang{
		Name:        request.Name,
		Nim:         request.Nim,
		NamaDosbing: request.NamaDosbing,
		ScheduledAt: request.ScheduledAt,
		NamaDosuji1: request.NamaDosuji1,
		NamaDosuji2: request.NamaDosuji2,
		Ruang:       request.Ruang,
		Judul:       request.Judul,
		IDBatch:     request.IDBatch,
	}

	fmt.Println(targetJadwalSidang)

	newJadwal, err := t.jadwalSidangUsecase.AddJadwalSidang(*targetJadwalSidang)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Jadwal added successfully", newJadwal))
}

// GetJadwalSidangByBatch retrieves jadwal sidang records by batch ID.
// @Summary Get jadwal sidang by batch
// @Description Retrieves jadwal sidang records by batch ID.
// @Tags Jadwal Sidang
// @Accept json
// @Produce json
// @Param id_batch query string true "Batch ID"
// @Success 200 {object} []entity.JadwalSidang
// @Router /jadwal-sidang-batch [get]
func (t *AdminJadwalSidangHandler) GetJadwalSidangByBatch(c echo.Context) error {
	idBatch := c.QueryParam("id_batch")
	sizeStr := c.QueryParam("limit")
	offsetStr := c.QueryParam("offset")

	size, err := strconv.Atoi(sizeStr)
	if err != nil {
		// Handle error, maybe return an error response
		size = 10
	}

	offset, err := strconv.Atoi(offsetStr)
	if err != nil {
		// Handle error, maybe return an error response
		offset = 0
	}

	jadwal, err := t.jadwalSidangUsecase.GetJadwalSidangByBatch(idBatch, size, offset)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Sidang successfully retreived", jadwal))
}

// DeleteJadwalSidang deletes a jadwal sidang record by ID.
// @Summary Delete jadwal sidang
// @Description Deletes a jadwal sidang record by ID.
// @Tags Jadwal Sidang
// @Accept json
// @Produce json
// @Param request body entity.EditJadwalSidangRequest true "Edit Jadwal Sidang Request"
// @Success 200 {object} entity.JadwalSidang
// @Router /jadwal-sidang [delete]
func (t *AdminJadwalSidangHandler) DeleteJadwalSidang(c echo.Context) error {
	request := new(entity.EditJadwalSidangRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	jadwal, err := t.jadwalSidangUsecase.DeleteJadwalSidang(request.ID)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Sidang successfully deleted", jadwal))
}

// EditJadwalSidang edits an existing jadwal sidang record.
// @Summary Edit jadwal sidang
// @Description Edits an existing jadwal sidang record.
// @Tags Jadwal Sidang
// @Accept json
// @Produce json
// @Param request body entity.EditJadwalSidangRequest true "Edit Jadwal Sidang Request"
// @Success 200 {object} entity.JadwalSidang
// @Router /jadwal-sidang [patch]
func (t *AdminJadwalSidangHandler) EditJadwalSidang(c echo.Context) error {
	request := new(entity.EditJadwalSidangRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	targetJadwalSidang := &entity.JadwalSidang{
		ID:          request.ID,
		Name:        request.Name,
		Nim:         request.Nim,
		NamaDosbing: request.NamaDosbing,
		ScheduledAt: request.ScheduledAt,
		NamaDosuji1: request.NamaDosuji1,
		NamaDosuji2: request.NamaDosuji2,
		Ruang:       request.Ruang,
		Judul:       request.Judul,
		IDBatch:     request.IDBatch,
	}

	updatedJadwalSidang, err := t.jadwalSidangUsecase.EditJadwalSidang(*targetJadwalSidang)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Sidang successfully edited", updatedJadwalSidang))
}

// GetJadwalSidangAll retrieves all jadwal sidang records.
// @Summary Get all jadwal sidang
// @Description Retrieves all jadwal sidang records.
// @Tags Jadwal Sidang
// @Accept json
// @Produce json
// @Success 200 {object} []entity.JadwalSidang
// @Router /jadwal-sidang [get]
func (t *AdminJadwalSidangHandler) GetJadwalSidangAll(c echo.Context) error {
	jadwal, err := t.jadwalSidangUsecase.GetJadwalSidangAll()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Sidang successfully retreived", jadwal))
}

// GetJadwalSidangInDateRange retrieves all jadwal sidang records in a date range.
// @Summary Get all jadwal sidang within a date range
// @Description Retrieves all jadwal sidang records within a date range.
// @Tags Jadwal Sidang
// @Accept json
// @Produce json
// @Success 200 {object} []entity.JadwalSidang
// @Router /jadwal-sidang-range [get]
func (t *AdminJadwalSidangHandler) GetJadwalSidangInDateRange(c echo.Context) error {
	startDate := c.QueryParam("startDate")
	endDate := c.QueryParam("endDate")

	jadwal, err := t.jadwalSidangUsecase.GetJadwalSidangInDateRange(startDate, endDate)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Sidang successfully retreived", jadwal))
}
